@@ -4,10 +4,9 @@ API for interacting with a CloudMan instance.
 import functools
 import json
 import time
+from urllib.parse import urlparse
 
 import requests
-from six.moves import range
-from six.moves.urllib.parse import urlparse
 
 import bioblend
 from bioblend.cloudman.launch import CloudManLauncher
@@ -60,7 +59,7 @@ class VMLaunchException(Exception):
         return repr(self.value)
 
 
-class CloudManConfig(object):
+class CloudManConfig:
 
     def __init__(self,
                  access_key=None,
@@ -74,7 +73,7 @@ class CloudManConfig(object):
                  galaxy_data_option='',
                  initial_storage_size=10,
                  key_name='cloudman_key_pair',
-                 security_groups=['CloudMan'],
+                 security_groups=None,
                  placement='',
                  kernel_id=None,
                  ramdisk_id=None,
@@ -161,6 +160,8 @@ class CloudManConfig(object):
                                  extra parameters to the ``CloudManInstance.launch_instance``
                                  method.
         """
+        if security_groups is None:
+            security_groups = ['CloudMan']
         self.set_connection_parameters(access_key, secret_key, cloud_metadata)
         self.set_pre_launch_parameters(
             cluster_name, image_id, instance_type,
@@ -177,8 +178,9 @@ class CloudManConfig(object):
     def set_pre_launch_parameters(
             self, cluster_name, image_id, instance_type, password,
             kernel_id=None, ramdisk_id=None, key_name='cloudman_key_pair',
-            security_groups=['CloudMan'], placement='',
-            block_until_ready=False):
+            security_groups=None, placement='', block_until_ready=False):
+        if security_groups is None:
+            security_groups = ['CloudMan']
         self.cluster_name = cluster_name
         self.image_id = image_id
         self.instance_type = instance_type
@@ -201,7 +203,7 @@ class CloudManConfig(object):
     class CustomTypeEncoder(json.JSONEncoder):
         def default(self, obj):
             if isinstance(obj, (CloudManConfig, Bunch)):
-                key = '__%s__' % obj.__class__.__name__
+                key = f"__{obj.__class__.__name__}__"
                 return {key: obj.__dict__}
             return json.JSONEncoder.default(self, obj)
 
@@ -235,16 +237,16 @@ class CloudManConfig(object):
         elif self.password is None:
             return "Password must not be null"
         elif self.cluster_type not in [None, 'Test', 'Data', 'Galaxy', 'Shared_cluster']:
-            return "Unrecognized cluster type ({0})".format(self.cluster_type)
+            return f"Unrecognized cluster type ({self.cluster_type})"
         elif self.galaxy_data_option not in [None, '', 'custom-size', 'transient']:
-            return "Unrecognized galaxy data option ({0})".format(self.galaxy_data_option)
+            return f"Unrecognized galaxy data option ({self.galaxy_data_option})"
         elif self.key_name is None:
             return "Key-pair name must not be null"
         else:
             return None
 
 
-class GenericVMInstance(object):
+class GenericVMInstance:
 
     def __init__(self, launcher, launch_result):
         """
@@ -334,16 +336,18 @@ class GenericVMInstance(object):
                 self._init_instance(status['public_ip'])
                 return
             elif status['error'] != '':
-                msg = "Error launching an instance: {0}".format(status['error'])
+                msg = f"Error launching an instance: {status['error']}"
                 bioblend.log.error(msg)
                 raise VMLaunchException(msg)
             else:
-                bioblend.log.warn("Instance not ready yet (it's in state '{0}'); waiting another {1} seconds..."
-                                  .format(status['instance_state'], time_left))
+                bioblend.log.warning(
+                    f"Instance not ready yet (it's in state '{status['instance_state']}'); waiting another {time_left} seconds..."
+                )
                 time.sleep(vm_ready_check_interval)
 
-        raise VMLaunchException("Waited too long for instance to become ready. Instance Id: %s"
-                                % self.instance_id)
+        raise VMLaunchException(
+            f"Waited too long for instance to become ready. Instance Id: {self.instance_id}"
+        )
 
 
 class CloudManInstance(GenericVMInstance):
@@ -359,19 +363,19 @@ class CloudManInstance(GenericVMInstance):
         """
         self.initialized = False
         if kwargs.get('launch_result', None) is not None:  # Used internally by the launch_instance method
-            super(CloudManInstance, self).__init__(kwargs['launcher'], kwargs['launch_result'])
+            super().__init__(kwargs['launcher'], kwargs['launch_result'])
         else:
-            super(CloudManInstance, self).__init__(None, None)
-        self.config = kwargs.pop('cloudman_config', None)
-        if not self.config:
-            self.password = password
-        else:
-            self.password = self.config.password
+            super().__init__(None, None)
+        self.config = kwargs.pop('cloudman_config', CloudManConfig())
+        self.password = password or self.config.password
+        self.use_ssl = kwargs.get("use_ssl", self.config.kwargs.get("use_ssl", False))
+        self.verify = kwargs.get("verify", self.config.kwargs.get("verify", False))
+        self.authuser = kwargs.get("authuser", "")
         self._set_url(url)
 
     def __repr__(self):
         if self.cloudman_url:
-            return "CloudMan instance at {0}".format(self.cloudman_url)
+            return f"CloudMan instance at {self.cloudman_url}"
         else:
             return "Waiting for this CloudMan instance to start..."
 
@@ -383,7 +387,7 @@ class CloudManInstance(GenericVMInstance):
         self._set_url(host_name)
 
     def _init_instance(self, hostname):
-        super(CloudManInstance, self)._init_instance(hostname)
+        super()._init_instance(hostname)
         if self.config.cluster_type:
             self.initialize(self.config.cluster_type, galaxy_data_option=self.config.galaxy_data_option, initial_storage_size=self.config.initial_storage_size)
 
@@ -392,13 +396,16 @@ class CloudManInstance(GenericVMInstance):
         Keeps the CloudMan URL as well and the hostname in sync.
         """
         if url:
-            parse_result = urlparse(url)
             # Make sure the URL scheme is defined (otherwise requests will not work)
-            if not parse_result.scheme:
-                url = "http://" + url
+            if not url.lower().startswith('http'):
+                # Check to see whether https scheme is required
+                if self.use_ssl:
+                    url = "https://" + url
+                else:
+                    url = "http://" + url
             # Parse the corrected URL again to extract the hostname
             parse_result = urlparse(url)
-            super(CloudManInstance, self)._update_host_name(parse_result.hostname)
+            super()._update_host_name(parse_result.hostname)
         self.url = url
 
     @property
@@ -415,7 +422,7 @@ class CloudManInstance(GenericVMInstance):
         Returns the URL for accessing this instance of CloudMan.
         """
         if self.url:
-            return '/'.join([self.url, 'cloud'])
+            return self.url + '/cloud'
         return None
 
     @staticmethod
@@ -430,15 +437,15 @@ class CloudManInstance(GenericVMInstance):
         validation_result = cfg.validate()
         if validation_result is not None:
             raise VMLaunchException(
-                "Invalid CloudMan configuration provided: {0}"
-                .format(validation_result))
+                f"Invalid CloudMan configuration provided: {validation_result}"
+            )
         launcher = CloudManLauncher(cfg.access_key, cfg.secret_key, cfg.cloud_metadata)
         result = launcher.launch(
             cfg.cluster_name, cfg.image_id, cfg.instance_type, cfg.password,
             cfg.kernel_id, cfg.ramdisk_id, cfg.key_name, cfg.security_groups,
             cfg.placement, **cfg.kwargs)
         if result['error'] is not None:
-            raise VMLaunchException("Error launching cloudman instance: {0}".format(result['error']))
+            raise VMLaunchException(f"Error launching cloudman instance: {result['error']}")
         instance = CloudManInstance(None, None, launcher=launcher,
                                     launch_result=result, cloudman_config=cfg)
         if cfg.block_until_ready and cfg.cluster_type:
@@ -478,7 +485,7 @@ class CloudManInstance(GenericVMInstance):
         try:
             r = self._make_get_request("cloudman_version")
             return r['version']
-        except:
+        except Exception:
             return 1
 
     @block_until_vm_ready
@@ -516,7 +523,7 @@ class CloudManInstance(GenericVMInstance):
         """
         Get the ``cluster type`` for this CloudMan instance. See the
         CloudMan docs about the available types. Returns a dictionary,
-        for example: ``{u'cluster_type': u'Test'}``.
+        for example: ``{'cluster_type': 'Test'}``.
         """
         cluster_type = self._make_get_request("cluster_type")
         if cluster_type['cluster_type']:
@@ -712,7 +719,7 @@ class CloudManInstance(GenericVMInstance):
                                         timeout=15)
         return result
 
-    def _make_get_request(self, url, parameters={}, timeout=None):
+    def _make_get_request(self, url, parameters=None, timeout=None):
         """
         Private function that makes a GET request to the nominated ``url``,
         with the provided GET ``parameters``. Optionally, set the ``timeout``
@@ -720,10 +727,18 @@ class CloudManInstance(GenericVMInstance):
         particularly useful when terminating a cluster as it may terminate
         before sending a response.
         """
-        req_url = '/'.join([self.cloudman_url, 'root', url])
-        r = requests.get(req_url, params=parameters, auth=("", self.password), timeout=timeout)
+        if parameters is None:
+            parameters = {}
+        req_url = '/'.join((self.cloudman_url, 'root', url))
+        r = requests.get(
+            req_url,
+            params=parameters,
+            auth=(self.authuser, self.password),
+            timeout=timeout,
+            verify=self.verify,
+        )
         try:
             json = r.json()
             return json
-        except:
+        except Exception:
             return r.text
