@@ -2,27 +2,28 @@
 Setup and launch a CloudMan instance.
 """
 import datetime
-import yaml
 import socket
+from http.client import (
+    BadStatusLine,
+    HTTPConnection,
+    HTTPException,
+)
+from urllib.parse import urlparse
 
 import boto
-from boto.compat import http_client
+import yaml
 from boto.ec2.regioninfo import RegionInfo
 from boto.exception import EC2ResponseError, S3ResponseError
 from boto.s3.connection import OrdinaryCallingFormat, S3Connection, SubdomainCallingFormat
-import six
-from six.moves.http_client import HTTPConnection
-from six.moves.urllib.parse import urlparse
 
 import bioblend
 from bioblend.util import Bunch
 
+
 # Uncomment the following line if no logging from boto is desired
 # bioblend.logging.getLogger('boto').setLevel(bioblend.logging.CRITICAL)
 # Uncomment the following line if logging at the prompt is desired
 # bioblend.set_stream_logger(__name__)
-
-
 def instance_types(cloud_name='generic'):
     """
     Return a list of dictionaries containing details about the available
@@ -67,7 +68,8 @@ def instance_types(cloud_name='generic'):
     return instance_list
 
 
-class CloudManLauncher(object):
+class CloudManLauncher:
+
     def __init__(self, access_key, secret_key, cloud=None):
         """
         Define the environment in which this instance of CloudMan will be launched.
@@ -100,17 +102,29 @@ class CloudManLauncher(object):
                                s3_conn_path='/')
         else:
             self.cloud = cloud
-        self.ec2_conn = self.connect_ec2(self.access_key, self.secret_key, self.cloud)
-        # Define exceptions from http_client that we want to catch and retry
-        self.http_exceptions = (http_client.HTTPException, socket.error,
-                                socket.gaierror, http_client.BadStatusLine)
+        self.ec2_conn = self.connect_ec2(
+            self.access_key,
+            self.secret_key,
+            self.cloud)
+        self.vpc_conn = self.connect_vpc(
+            self.access_key,
+            self.secret_key,
+            self.cloud)
+        # Define exceptions that we want to catch and retry
+        self.http_exceptions = (
+            HTTPException,
+            socket.error,
+            socket.gaierror,
+            BadStatusLine
+        )
 
     def __repr__(self):
-        return "Cloud: {0}; acct ID: {1}".format(self.cloud.name, self.access_key)
+        return f"Cloud: {self.cloud.name}; acct ID: {self.access_key}"
 
     def launch(self, cluster_name, image_id, instance_type, password,
                kernel_id=None, ramdisk_id=None, key_name='cloudman_key_pair',
-               security_groups=['CloudMan'], placement='', subnet_id=None, **kwargs):
+               security_groups=None, placement='', subnet_id=None,
+               ebs_optimized=False, **kwargs):
         """
         Check all the prerequisites (key pair and security groups) for
         launching a CloudMan instance, compose the user data based on the
@@ -118,7 +132,7 @@ class CloudManLauncher(object):
         defined in the object's ``cloud`` field.
 
         For the current list of user data fields that can be provided via
-        ``kwargs``, see `<http://wiki.g2.bx.psu.edu/CloudMan/UserData>`_
+        ``kwargs``, see `<https://galaxyproject.org/cloudman/userdata/>`_
 
         Return a dict containing the properties and info with which an instance
         was launched, namely: ``sg_names`` containing the names of the security
@@ -131,6 +145,8 @@ class CloudManLauncher(object):
         ``instance_id`` containing the ID of a started instance, and
         ``error`` containing an error message if there was one.
         """
+        if security_groups is None:
+            security_groups = ['CloudMan']
         ret = {'sg_names': [],
                'sg_ids': [],
                'kp_name': '',
@@ -138,29 +154,47 @@ class CloudManLauncher(object):
                'rs': None,
                'instance_id': '',
                'error': None}
-        security_group_ids = []
         # First satisfy the prerequisites
         for sg in security_groups:
-            cmsg = self.create_cm_security_group(sg)
+            # Get VPC ID in case we're launching into a VPC
+            vpc_id = None
+            if subnet_id:
+                try:
+                    sn = self.vpc_conn.get_all_subnets(subnet_id)[0]
+                    vpc_id = sn.vpc_id
+                except (EC2ResponseError, IndexError):
+                    bioblend.log.exception("Trouble fetching subnet %s", subnet_id)
+            cmsg = self.create_cm_security_group(sg, vpc_id=vpc_id)
             ret['error'] = cmsg['error']
             if ret['error']:
                 return ret
             if cmsg['name']:
                 ret['sg_names'].append(cmsg['name'])
                 ret['sg_ids'].append(cmsg['sg_id'])
-                security_group_ids.append(cmsg['sg_id'])
+                if subnet_id:
+                    # Must setup a network interface if launching into VPC
+                    security_groups = None
+                    interface = boto.ec2.networkinterface.NetworkInterfaceSpecification(
+                        subnet_id=subnet_id, groups=[cmsg['sg_id']],
+                        associate_public_ip_address=True)
+                    network_interfaces = (boto.ec2.networkinterface.
+                                          NetworkInterfaceCollection(interface))
+                else:
+                    network_interfaces = None
         kp_info = self.create_key_pair(key_name)
+        ret['kp_name'] = kp_info['name']
+        ret['kp_material'] = kp_info['material']
         ret['error'] = kp_info['error']
         if ret['error']:
             return ret
-        ret['kp_name'] = kp_info['name']
-        ret['kp_material'] = kp_info['material']
         # If not provided, try to find a placement
         # TODO: Should placement always be checked? To make sure it's correct
         # for existing clusters.
         if not placement:
-            placement = self._find_placement(cluster_name).get('placement', None)
-        # Compose user data for launching an instance, ensuring we have the required fields
+            placement = self._find_placement(
+                cluster_name).get('placement', None)
+        # Compose user data for launching an instance, ensuring we have the
+        # required fields
         kwargs['access_key'] = self.access_key
         kwargs['secret_key'] = self.secret_key
         kwargs['cluster_name'] = cluster_name
@@ -169,41 +203,45 @@ class CloudManLauncher(object):
         ud = self._compose_user_data(kwargs)
         # Now launch an instance
         try:
+
             rs = None
             rs = self.ec2_conn.run_instances(image_id=image_id,
                                              instance_type=instance_type,
                                              key_name=key_name,
-                                             security_group_ids=security_group_ids,
+                                             security_groups=security_groups,
+                                             # The following two arguments are
+                                             # provided in the network_interface
+                                             # instead of arguments:
+                                             # security_group_ids=security_group_ids,
+                                             # subnet_id=subnet_id,
+                                             network_interfaces=network_interfaces,
                                              user_data=ud,
                                              kernel_id=kernel_id,
                                              ramdisk_id=ramdisk_id,
-                                             subnet_id=subnet_id,
-                                             placement=placement)
+                                             placement=placement,
+                                             ebs_optimized=ebs_optimized)
             ret['rs'] = rs
         except EC2ResponseError as e:
-            err_msg = "Problem launching an instance: {0} (code {1}; status {2})" \
-                      .format(e.message, e.error_code, e.status)
+            err_msg = f"Problem launching an instance: {e} (code {e.error_code}; status {e.status})"
             bioblend.log.exception(err_msg)
             ret['error'] = err_msg
             return ret
         else:
             if rs:
                 try:
-                    bioblend.log.info("Launched an instance with ID %s" % rs.instances[0].id)
+                    bioblend.log.info("Launched an instance with ID %s", rs.instances[0].id)
                     ret['instance_id'] = rs.instances[0].id
                     ret['instance_ip'] = rs.instances[0].ip_address
                 except EC2ResponseError as e:
-                    err_msg = "Problem with the launched instance object: {0} " \
-                              "(code {1}; status {2})" \
-                              .format(e.message, e.error_code, e.status)
+                    err_msg = f"Problem with the launched instance object: {e} (code {e.error_code}; status {e.status})"
                     bioblend.log.exception(err_msg)
                     ret['error'] = err_msg
             else:
-                    ret['error'] = ("No response after launching an instance. Check "
-                                    "your account permissions and try again.")
+                ret['error'] = ("No response after launching an instance. Check "
+                                "your account permissions and try again.")
         return ret
 
-    def create_cm_security_group(self, sg_name='CloudMan'):
+    def create_cm_security_group(self, sg_name='CloudMan', vpc_id=None):
         """
         Create a security group with all authorizations required to run CloudMan.
 
@@ -212,6 +250,9 @@ class CloudManLauncher(object):
         :type sg_name: str
         :param sg_name: A name for the security group to be created.
 
+        :type vpc_id: str
+        :param vpc_id: VPC ID under which to create the security group.
+
         :rtype: dict
         :return: A dictionary containing keys ``name`` (with the value being the
                  name of the security group that was created), ``error``
@@ -235,33 +276,31 @@ class CloudManLauncher(object):
                     'error': None,
                     'ports': ports}
         cmsg = None
+        filters = None
+        if vpc_id:
+            filters = {'vpc-id': vpc_id}
         # Check if this security group already exists
         try:
-            sgs = self.ec2_conn.get_all_security_groups()
+            sgs = self.ec2_conn.get_all_security_groups(filters=filters)
         except EC2ResponseError as e:
-            err_msg = ("Problem getting security groups. This could indicate a "
-                       "problem with your account credentials or permissions: "
-                       "{0} (code {1}; status {2})"
-                       .format(e.message, e.error_code, e.status))
+            err_msg = f"Problem getting security groups. This could indicate a problem with your account credentials or permissions: {e} (code {e.error_code}; status {e.status})"
             bioblend.log.exception(err_msg)
             progress['error'] = err_msg
             return progress
         for sg in sgs:
             if sg.name == sg_name:
                 cmsg = sg
-                bioblend.log.debug("Security group '%s' already exists; will "
-                                   "add authorizations next." % sg_name)
+                bioblend.log.debug("Security group '%s' already exists; will add authorizations next.", sg_name)
                 break
         # If it does not exist, create security group
         if cmsg is None:
-            bioblend.log.debug("Creating Security Group %s" % sg_name)
+            bioblend.log.debug("Creating Security Group %s", sg_name)
             try:
                 cmsg = self.ec2_conn.create_security_group(sg_name, 'A security '
-                                                           'group for CloudMan')
+                                                           'group for CloudMan',
+                                                           vpc_id=vpc_id)
             except EC2ResponseError as e:
-                err_msg = "Problem creating security group '{0}': {1} (code {2}; " \
-                          "status {3})" \
-                          .format(sg_name, e.message, e.error_code, e.status)
+                err_msg = f"Problem creating security group '{sg_name}': {e} (code {e.error_code}; status {e.status})"
                 bioblend.log.exception(err_msg)
                 progress['error'] = err_msg
         if cmsg:
@@ -271,52 +310,66 @@ class CloudManLauncher(object):
             # If these rules already exist, nothing will be changed in the SG
             for port in ports:
                 try:
-                    if not self.rule_exists(cmsg.rules, from_port=port[0], to_port=port[1]):
-                        cmsg.authorize(ip_protocol='tcp', from_port=port[0], to_port=port[1], cidr_ip='0.0.0.0/0')
+                    if not self.rule_exists(
+                            cmsg.rules, from_port=port[0], to_port=port[1]):
+                        cmsg.authorize(
+                            ip_protocol='tcp',
+                            from_port=port[0],
+                            to_port=port[1],
+                            cidr_ip='0.0.0.0/0')
                     else:
-                        bioblend.log.debug("Rule (%s:%s) already exists in the SG" % (port[0], port[1]))
+                        bioblend.log.debug("Rule (%s:%s) already exists in the SG", port[0], port[1])
                 except EC2ResponseError as e:
-                    err_msg = "A problem adding security group authorizations: {0} " \
-                              "(code {1}; status {2})" \
-                              .format(e.message, e.error_code, e.status)
+                    err_msg = f"A problem adding security group authorizations: {e} (code {e.error_code}; status {e.status})"
                     bioblend.log.exception(err_msg)
                     progress['error'] = err_msg
             # Add ICMP (i.e., ping) rule required by HTCondor
             try:
-                if not self.rule_exists(cmsg.rules, from_port='-1', to_port='-1', ip_protocol='icmp'):
-                    cmsg.authorize(ip_protocol='icmp', from_port=-1, to_port=-1, cidr_ip='0.0.0.0/0')
+                if not self.rule_exists(
+                        cmsg.rules, from_port='-1', to_port='-1', ip_protocol='icmp'):
+                    cmsg.authorize(
+                        ip_protocol='icmp',
+                        from_port=-1,
+                        to_port=-1,
+                        cidr_ip='0.0.0.0/0')
                 else:
-                    bioblend.log.debug("ICMP rule already exists in {0} SG.".format(sg_name))
+                    bioblend.log.debug(
+                        f"ICMP rule already exists in {sg_name} SG.")
             except EC2ResponseError as e:
-                err_msg = "A problem with security ICMP rule authorization: {0} " \
-                          "(code {1}; status {2})" \
-                          .format(e.message, e.error_code, e.status)
+                err_msg = f"A problem with security ICMP rule authorization: {e} (code {e.error_code}; status {e.status})"
                 bioblend.log.exception(err_msg)
                 progress['err_msg'] = err_msg
-            # Add rule that allows communication between instances in the same SG
-            g_rule_exists = False  # A flag to indicate if group rule already exists
+            # Add rule that allows communication between instances in the same
+            # SG
+            # A flag to indicate if group rule already exists
+            g_rule_exists = False
             for rule in cmsg.rules:
                 for grant in rule.grants:
                     if grant.name == cmsg.name:
                         g_rule_exists = True
-                        bioblend.log.debug("Group rule already exists in the SG.")
+                        bioblend.log.debug(
+                            "Group rule already exists in the SG.")
                 if g_rule_exists:
                     break
             if not g_rule_exists:
                 try:
-                    cmsg.authorize(src_group=cmsg, ip_protocol='tcp', from_port=0, to_port=65535)
+                    cmsg.authorize(
+                        src_group=cmsg,
+                        ip_protocol='tcp',
+                        from_port=0,
+                        to_port=65535)
                 except EC2ResponseError as e:
-                    err_msg = "A problem with security group authorization: {0} " \
-                              "(code {1}; status {2})" \
-                              .format(e.message, e.error_code, e.status)
+                    err_msg = f"A problem with security group group authorization: {e} (code {e.error_code}; status {e.status})"
                     bioblend.log.exception(err_msg)
                     progress['err_msg'] = err_msg
-            bioblend.log.info("Done configuring '%s' security group" % cmsg.name)
+            bioblend.log.info("Done configuring '%s' security group", cmsg.name)
         else:
-            bioblend.log.warning("Did not create security group '{0}'".format(sg_name))
+            bioblend.log.warning(
+                f"Did not create security group '{sg_name}'")
         return progress
 
-    def rule_exists(self, rules, from_port, to_port, ip_protocol='tcp', cidr_ip='0.0.0.0/0'):
+    def rule_exists(
+            self, rules, from_port, to_port, ip_protocol='tcp', cidr_ip='0.0.0.0/0'):
         """
         A convenience method to check if an authorization rule in a security group
         already exists.
@@ -354,29 +407,37 @@ class CloudManLauncher(object):
         try:
             kps = self.ec2_conn.get_all_key_pairs()
         except EC2ResponseError as e:
-            err_msg = "Problem getting key pairs: {0} (code {1}; status {2})" \
-                      .format(e.message, e.error_code, e.status)
+            err_msg = f"Problem getting key pairs: {e} (code {e.error_code}; status {e.status})"
             bioblend.log.exception(err_msg)
             progress['error'] = err_msg
             return progress
         for akp in kps:
             if akp.name == key_name:
-                bioblend.log.info("Key pair '%s' already exists; reusing it." % key_name)
+                bioblend.log.info("Key pair '%s' already exists; reusing it.", key_name)
                 progress['name'] = akp.name
                 return progress
         try:
             kp = self.ec2_conn.create_key_pair(key_name)
         except EC2ResponseError as e:
-            err_msg = "Problem creating key pair '{0}': {1} (code {2}; status {3})" \
-                      .format(key_name, e.message, e.error_code, e.status)
+            err_msg = f"Problem creating key pair '{key_name}': {e} (code {e.error_code}; status {e.status})"
             bioblend.log.exception(err_msg)
             progress['error'] = err_msg
             return progress
-        bioblend.log.info("Created key pair '%s'" % kp.name)
+        bioblend.log.info("Created key pair '%s'", kp.name)
         progress['name'] = kp.name
         progress['material'] = kp.material
         return progress
 
+    def assign_floating_ip(self, ec2_conn, instance):
+        try:
+            bioblend.log.debug("Allocating a new floating IP address.")
+            address = ec2_conn.allocate_address()
+        except EC2ResponseError:
+            bioblend.log.exception("Exception allocating a new floating IP address")
+        bioblend.log.info("Associating floating IP %s to instance %s", address.public_ip, instance.id)
+        ec2_conn.associate_address(instance_id=instance.id,
+                                   public_ip=address.public_ip)
+
     def get_status(self, instance_id):
         """
         Check on the status of an instance. ``instance_id`` needs to be a
@@ -419,8 +480,17 @@ class CloudManLauncher(object):
                 public_ip = rs[0].instances[0].ip_address
                 state['public_ip'] = public_ip
                 if inst_state == 'running':
-                    cm_url = "http://{dns}/cloud".format(dns=public_ip)
-                    # Wait until the CloudMan URL is accessible to return the data
+                    # if there's a private ip, but no public ip
+                    # attempt auto allocation of floating IP
+                    if rs[0].instances[0].private_ip_address and not public_ip:
+                        self.assign_floating_ip(ec2_conn, rs[0].instances[0])
+                    # Wait until the CloudMan URL is accessible to return
+                    # the data - this may only be correct initially at
+                    # bootup for instances that configure themselves for
+                    # https - they may ultimately block port 80. However,
+                    # we don't have a good way to determine whether to
+                    # check using http or https.
+                    cm_url = f"http://{public_ip}/cloud"
                     if self._checkURL(cm_url) is True:
                         state['instance_state'] = inst_state
                         state['placement'] = rs[0].instances[0].placement
@@ -429,7 +499,7 @@ class CloudManLauncher(object):
                 else:
                     state['instance_state'] = inst_state
         except Exception as e:
-            err = "Problem updating instance '%s' state: %s" % (instance_id, e)
+            err = f"Problem updating instance '{instance_id}' state: {e}"
             bioblend.log.error(err)
             state['error'] = err
         return state
@@ -463,9 +533,9 @@ class CloudManLauncher(object):
         try:
             buckets = s3_conn.get_all_buckets()
         except S3ResponseError as e:
-            response['error'] = "S3ResponseError getting buckets: %s" % e
+            response['error'] = f"S3ResponseError getting buckets: {e}"
         except self.http_exceptions as ex:
-            response['error'] = "Exception getting buckets: %s" % ex
+            response['error'] = f"Exception getting buckets: {ex}"
         if response['error']:
             bioblend.log.exception(response['error'])
             return response
@@ -476,8 +546,7 @@ class CloudManLauncher(object):
             except S3ResponseError:
                 # This can fail for a number of reasons for non-us and/or
                 # CNAME'd buckets but it is not a terminal error
-                bioblend.log.warning("Problem fetching persistent_data.yaml "
-                                     "from bucket %s" % bucket)
+                bioblend.log.warning("Problem fetching persistent_data.yaml from bucket %s", bucket)
                 continue
             if pd:
                 # We are dealing with a CloudMan bucket
@@ -529,8 +598,6 @@ class CloudManLauncher(object):
         r = RegionInfo(name=ci['region_name'], endpoint=ci['region_endpoint'])
         ec2_conn = boto.connect_ec2(aws_access_key_id=a_key,
                                     aws_secret_access_key=s_key,
-                                    # api_version is needed for availability zone support for EC2
-                                    api_version='2012-06-01' if ci['cloud_type'] == 'ec2' else None,
                                     is_secure=ci['is_secure'],
                                     region=r,
                                     port=ci['ec2_port'],
@@ -558,6 +625,26 @@ class CloudManLauncher(object):
             path=ci['s3_conn_path'], calling_format=calling_format)
         return s3_conn
 
+    def connect_vpc(self, a_key, s_key, cloud=None):
+        """
+        Establish a connection to the VPC service.
+
+        TODO: Make this work with non-default clouds as well.
+        """
+        if cloud is None:
+            cloud = self.cloud
+        ci = self._get_cloud_info(cloud)
+        r = RegionInfo(name=ci['region_name'], endpoint=ci['region_endpoint'])
+        vpc_conn = boto.connect_vpc(
+            aws_access_key_id=a_key,
+            aws_secret_access_key=s_key,
+            is_secure=ci['is_secure'],
+            region=r,
+            port=ci['ec2_port'],
+            path=ci['ec2_conn_path'],
+            validate_certs=False)
+        return vpc_conn
+
     def _compose_user_data(self, user_provided_data):
         """
         A convenience method used to compose and properly format the user data
@@ -572,17 +659,23 @@ class CloudManLauncher(object):
         excluded_fields = ['sg_name', 'image_id', 'instance_id', 'kp_name',
                            'cloud', 'cloud_type', 'public_dns', 'cidr_range',
                            'kp_material', 'placement', 'flavor_id']
-        for key, value in six.iteritems(user_provided_data):
+        for key, value in user_provided_data.items():
             if key not in excluded_fields:
                 form_data[key] = value
-        # If the following user data keys are empty, do not include them in the request user data
-        udkeys = ['post_start_script_url', 'worker_post_start_script_url', 'bucket_default', 'share_string']
+        # If the following user data keys are empty, do not include them in the
+        # request user data
+        udkeys = [
+            'post_start_script_url',
+            'worker_post_start_script_url',
+            'bucket_default',
+            'share_string']
         for udkey in udkeys:
             if udkey in form_data and form_data[udkey] == '':
                 del form_data[udkey]
         # If bucket_default was not provided, add a default value to the user data
         # (missing value does not play nicely with CloudMan's ec2autorun.py)
-        if not form_data.get('bucket_default', None) and self.cloud.bucket_default:
+        if not form_data.get(
+                'bucket_default', None) and self.cloud.bucket_default:
             form_data['bucket_default'] = self.cloud.bucket_default
         # Reuse the ``password`` for the ``freenxpass`` user data option
         if 'freenxpass' not in form_data and 'password' in form_data:
@@ -621,13 +714,12 @@ class CloudManLauncher(object):
         try:
             vol = self.ec2_conn.get_all_volumes(volume_ids=[vol_id])
         except EC2ResponseError as ec2e:
-            bioblend.log.error("EC2ResponseError querying for volume {0}: {1}"
-                               .format(vol_id, ec2e))
+            bioblend.log.error(f"EC2ResponseError querying for volume {vol_id}: {ec2e}")
             vol = None
         if vol:
             return vol[0].zone
         else:
-            bioblend.log.error("Requested placement of a volume '%s' that does not exist." % vol_id)
+            bioblend.log.error("Requested placement of a volume '%s' that does not exist.", vol_id)
             return None
 
     def _find_placement(self, cluster_name, cluster=None):
@@ -662,25 +754,28 @@ class CloudManLauncher(object):
                     response['placement'] = self._get_volume_placement(vol_id)
                 elif 'filesystems' in pd:
                     # V2 format.
-                    for fs in [fs for fs in pd['filesystems'] if fs.get('kind', None) == 'volume' and 'ids' in fs]:
-                        vol_id = fs['ids'][0]  # All volumes must be in the same zone
-                        response['placement'] = self._get_volume_placement(vol_id)
+                    for fs in [fs for fs in pd['filesystems'] if fs.get(
+                            'kind', None) == 'volume' and 'ids' in fs]:
+                        # All volumes must be in the same zone
+                        vol_id = fs['ids'][0]
+                        response['placement'] = self._get_volume_placement(
+                            vol_id)
                         # No need to continue to iterate through
                         # filesystems, if we found one with a volume.
                         break
             except Exception as exc:
-                response['error'] = ("Exception while finding placement for "
-                                     "cluster '{0}'. This can indicate malformed "
-                                     "instance data. Or that this method is "
-                                     "broken: {1}".format(cluster_name, exc))
+                response['error'] = \
+                    f"Exception while finding placement for cluster '{cluster_name}'. This can indicate malformed instance data. Or that this method is broken: {exc}"
                 bioblend.log.error(response['error'])
                 response['placement'] = None
         else:
-            bioblend.log.debug("Insufficient info about cluster {0} to get placement."
-                               .format(cluster_name))
+            bioblend.log.debug(
+                f"Insufficient info about cluster {cluster_name} to get placement."
+            )
         return response
 
-    def find_placements(self, ec2_conn, instance_type, cloud_type, cluster_name=None):
+    def find_placements(
+            self, ec2_conn, instance_type, cloud_type, cluster_name=None):
         """
         Find a list of placement zones that support the specified instance type.
 
@@ -720,7 +815,8 @@ class CloudManLauncher(object):
         if not response['zones']:
             in_the_past = datetime.datetime.now() - datetime.timedelta(hours=1)
             back_compatible_zone = "us-east-1e"
-            for zone in [z for z in ec2_conn.get_all_zones() if z.state == 'available']:
+            for zone in [
+                    z for z in ec2_conn.get_all_zones() if z.state == 'available']:
                 # Non EC2 clouds may not support get_spot_price_history
                 if instance_type is None or cloud_type != 'ec2':
                     zones.append(zone.name)
@@ -728,12 +824,13 @@ class CloudManLauncher(object):
                                                      end_time=in_the_past.isoformat(),
                                                      availability_zone=zone.name):
                     zones.append(zone.name)
-            zones.sort(reverse=True)  # Higher-lettered zones seem to have more availability currently
+            # Higher-lettered zones seem to have more availability currently
+            zones.sort(reverse=True)
             if back_compatible_zone in zones:
-                zones = [back_compatible_zone] + [z for z in zones if z != back_compatible_zone]
+                zones = [back_compatible_zone] + \
+                    [z for z in zones if z != back_compatible_zone]
             if len(zones) == 0:
-                response['error'] = ("Did not find availabilty zone for {1}"
-                                     .format(instance_type))
+                response['error'] = f"Did not find availabilty zone for {instance_type}"
                 bioblend.log.error(response['error'])
                 zones.append(back_compatible_zone)
         return response
@@ -749,7 +846,8 @@ class CloudManLauncher(object):
             h.putrequest('HEAD', p[2])
             h.endheaders()
             r = h.getresponse()
-            if r.status in (200, 401):  # CloudMan UI is pwd protected so include 401
+            # CloudMan UI is pwd protected so include 401
+            if r.status in (200, 401):
                 return True
         except Exception:
             # No response or no good response
