@@ -1,20 +1,29 @@
 """
 Contains possible interactions with the Galaxy Data Libraries
 """
+import logging
+import time
+import warnings
+
 from bioblend.galaxy.client import Client
+from bioblend.galaxy.datasets import (
+    DatasetTimeoutException,
+    TERMINAL_STATES,
+)
 from bioblend.util import attach_file
 
+log = logging.getLogger(__name__)
+
 
 class LibraryClient(Client):
+    module = 'libraries'
 
     def __init__(self, galaxy_instance):
-        self.module = 'libraries'
-        super(LibraryClient, self).__init__(galaxy_instance)
+        super().__init__(galaxy_instance)
 
     def create_library(self, name, description=None, synopsis=None):
         """
         Create a data library with the properties defined in the arguments.
-        Return a list of JSON dicts, looking like so::
 
         :type name: str
         :param name: Name of the new data library
@@ -26,41 +35,43 @@ class LibraryClient(Client):
         :param synopsis: Optional data library synopsis
 
         :rtype: dict
-        :return: details of the created library:
-
-            {"id": "f740ab636b360a70",
-              "name": "Library from bioblend",
-              "url": "/api/libraries/f740ab636b360a70"}
+        :return: Details of the created library.
+          For example::
 
+            {'id': 'f740ab636b360a70',
+             'name': 'Library from bioblend',
+             'url': '/api/libraries/f740ab636b360a70'}
         """
         payload = {'name': name}
         if description:
             payload['description'] = description
         if synopsis:
             payload['synopsis'] = synopsis
-        return Client._post(self, payload)
+        return self._post(payload)
 
     def delete_library(self, library_id):
         """
         Delete a data library.
 
         :type library_id: str
-        :param library_id: Encoded data library ID identifying the library to be deleted
+        :param library_id: Encoded data library ID identifying the library to be
+          deleted
+
+        :rtype: dict
+        :return: Information about the deleted library
 
         .. warning::
-            Deleting a data library is irreversible - all of the data from
-            the library will be permanently deleted.
+          Deleting a data library is irreversible - all of the data from the
+          library will be permanently deleted.
         """
-        payload = {}
-        return Client._delete(self, payload, id=library_id)
+        return self._delete(id=library_id)
 
-    def __show_item(self, library_id, item_id):
+    def _show_item(self, library_id, item_id):
         """
         Get details about a given library item.
         """
-        url = self.gi._make_url(self, library_id, contents=True)
-        url = '/'.join([url, item_id])
-        return Client._get(self, url=url)
+        url = '/'.join((self._make_url(library_id, contents=True), item_id))
+        return self._get(url=url)
 
     def delete_library_dataset(self, library_id, dataset_id, purged=False):
         """
@@ -73,19 +84,48 @@ class LibraryClient(Client):
         :param dataset_id: id of the dataset to be deleted
 
         :type purged: bool
-        :param purged: Indicate that the dataset should be purged (permanently deleted)
+        :param purged: Indicate that the dataset should be purged (permanently
+          deleted)
 
         :rtype: dict
-        :return: A dictionary containing the dataset id and whether the dataset has been deleted
-                 For example::
+        :return: A dictionary containing the dataset id and whether the dataset
+          has been deleted.
+          For example::
 
-                 {u'deleted': True, u'id': u'60e680a037f41974'}
+            {'deleted': True,
+             'id': '60e680a037f41974'}
         """
+        url = '/'.join((self._make_url(library_id, contents=True), dataset_id))
+        return self._delete(payload={'purged': purged}, url=url)
 
-        url = self.gi._make_url(self, library_id, contents=True)
-        # Append the dataset_id to the base history contents URL
-        url = '/'.join([url, dataset_id])
-        return Client._delete(self, url=url, payload={'purged': purged})
+    def update_library_dataset(self, dataset_id, **kwds):
+        """
+        Update library dataset metadata. Some of the attributes that can be
+        modified are documented below.
+
+        :type dataset_id: str
+        :param dataset_id: id of the dataset to be deleted
+
+        :type name: str
+        :param name: Replace library dataset name with the given string
+
+        :type misc_info: str
+        :param misc_info: Replace library dataset misc_info with given string
+
+        :type file_ext: str
+        :param file_ext: Replace library dataset extension (must exist in the Galaxy registry)
+
+        :type genome_build: str
+        :param genome_build: Replace library dataset genome build (dbkey)
+
+        :type tags: list
+        :param tags: Replace library dataset tags with the given list
+
+        :rtype: dict
+        :return: details of the updated dataset
+        """
+        url = '/'.join((self._make_url(), 'datasets', dataset_id))
+        return self._patch(payload=kwds, url=url)
 
     def show_dataset(self, library_id, dataset_id):
         """
@@ -99,22 +139,65 @@ class LibraryClient(Client):
         :param dataset_id: id of the dataset to be inspected
 
         :rtype: dict
-        :return: A dictionary containing information about the dataset in the library
+        :return: A dictionary containing information about the dataset in the
+          library
         """
-        return self.__show_item(library_id, dataset_id)
+        return self._show_item(library_id, dataset_id)
+
+    def wait_for_dataset(self, library_id, dataset_id, maxwait=12000, interval=3):
+        """
+        Wait until the library dataset state is terminal ('ok', 'empty',
+        'error', 'discarded' or 'failed_metadata').
+
+        :type library_id: str
+        :param library_id: library id where dataset is found in
+
+        :type dataset_id: str
+        :param dataset_id: id of the dataset to wait for
+
+        :type maxwait: float
+        :param maxwait: Total time (in seconds) to wait for the dataset state to
+          become terminal. If the dataset state is not terminal within this
+          time, a ``DatasetTimeoutException`` will be thrown.
+
+        :type interval: float
+        :param interval: Time (in seconds) to wait between 2 consecutive checks.
+
+        :rtype: dict
+        :return: A dictionary containing information about the dataset in the
+          library
+        """
+        assert maxwait >= 0
+        assert interval > 0
+
+        time_left = maxwait
+        while True:
+            dataset = self.show_dataset(library_id, dataset_id)
+            state = dataset['state']
+            if state in TERMINAL_STATES:
+                return dataset
+            if time_left > 0:
+                log.info("Dataset %s in library %s is in non-terminal state %s. Will wait %i more s", dataset_id, library_id, state, time_left)
+                time.sleep(min(time_left, interval))
+                time_left -= interval
+            else:
+                raise DatasetTimeoutException(f"Waited too long for dataset {dataset_id} in library {library_id} to complete")
 
     def show_folder(self, library_id, folder_id):
         """
-        Get details about a given folder. The required ``folder_id``
-        can be obtained from the folder's library content details.
+        Get details about a given folder. The required ``folder_id`` can be
+        obtained from the folder's library content details.
 
         :type library_id: str
         :param library_id: library id to inspect folders in
 
         :type folder_id: str
         :param folder_id: id of the folder to be inspected
+
+        :rtype: dict
+        :return: Information about the folder
         """
-        return self.__show_item(library_id, folder_id)
+        return self._show_item(library_id, folder_id)
 
     def _get_root_folder_id(self, library_id):
         """
@@ -124,14 +207,7 @@ class LibraryClient(Client):
         :param library_id: library id to find root of
         """
         l = self.show_library(library_id=library_id)
-        if 'root_folder_id' in l:
-            return l['root_folder_id']
-        # Galaxy previous to release_13.04 does not have root_folder_id in
-        # library dictionary, so resort to find the folder with name '/'
-        library_contents = self.show_library(library_id=library_id, contents=True)
-        for f in library_contents:
-            if f['name'] == '/':
-                return f['id']
+        return l['root_folder_id']
 
     def create_folder(self, library_id, folder_name, description=None, base_folder_id=None):
         """
@@ -149,6 +225,9 @@ class LibraryClient(Client):
         :type base_folder_id: str
         :param base_folder_id: id of the folder where to create the new folder.
           If not provided, the root folder will be used
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the new folder
         """
         # Get root folder ID if no ID was provided
         if base_folder_id is None:
@@ -160,25 +239,36 @@ class LibraryClient(Client):
         payload['create_type'] = 'folder'
         if description is not None:
             payload['description'] = description
-        return Client._post(self, payload, id=library_id, contents=True)
+        return self._post(payload, id=library_id, contents=True)
 
     def get_folders(self, library_id, folder_id=None, name=None):
         """
-        Get all the folders or filter specific one(s) via the provided ``name``
-        or ``folder_id`` in data library with id ``library_id``. Provide only one
-        argument: ``name`` or ``folder_id``, but not both.
+        Get all the folders in a library, or select a subset by specifying a
+        folder name for filtering.
+
+        :type library_id: str
+        :param library_id: library id to use
 
         :type folder_id: str
         :param folder_id: filter for folder by folder id
 
+          .. deprecated:: 0.16.0
+             To get details of a folder for which you know the ID, use the much
+             more efficient :meth:`show_folder` instead.
+
         :type name: str
-        :param name: filter for folder by name. For ``name`` specify the full
+        :param name: Folder name to filter on. For ``name`` specify the full
                      path of the folder starting from the library's root
                      folder, e.g. ``/subfolder/subsubfolder``.
 
         :rtype: list
-        :return: list of dicts each containing basic information about a folder.
+        :return: list of dicts each containing basic information about a folder
         """
+        if folder_id is not None:
+            warnings.warn(
+                'The folder_id parameter is deprecated, use the show_folder() method to view details of a folder for which you know the ID.',
+                category=FutureWarning
+            )
         if folder_id is not None and name is not None:
             raise ValueError('Provide only one argument between name or folder_id, but not both')
         library_contents = self.show_library(library_id=library_id, contents=True)
@@ -193,27 +283,35 @@ class LibraryClient(Client):
 
     def get_libraries(self, library_id=None, name=None, deleted=False):
         """
-        Get all the libraries or filter for specific one(s) via the provided name or ID.
-        Provide only one argument: ``name`` or ``library_id``, but not both.
+        Get all libraries, or select a subset by specifying optional arguments
+        for filtering (e.g. a library name).
 
         :type library_id: str
         :param library_id: filter for library by library id
 
+          .. deprecated:: 0.16.0
+             To get details of a library for which you know the ID, use the much
+             more efficient :meth:`show_library` instead.
+
         :type name: str
-        :param name: If ``name`` is set and multiple names match the given
-                     name, all the libraries matching the argument will be
-                     returned.
+        :param name: Library name to filter on.
 
         :type deleted: bool
-        :param deleted: If set to ``True``, return libraries that have been
-                        deleted.
+        :param deleted: If ``False`` (the default), return only non-deleted
+          libraries. If ``True``, return only deleted libraries. If ``None``,
+          return both deleted and non-deleted libraries.
 
         :rtype: list
-        :return: list of dicts each containing basic information about a library.
+        :return: list of dicts each containing basic information about a library
         """
+        if library_id is not None:
+            warnings.warn(
+                'The library_id parameter is deprecated, use the show_library() method to view details of a library for which you know the ID.',
+                category=FutureWarning
+            )
         if library_id is not None and name is not None:
             raise ValueError('Provide only one argument between name or library_id, but not both')
-        libraries = Client._get(self, deleted=deleted)
+        libraries = self._get(params={"deleted": deleted})
         if library_id is not None:
             library = next((_ for _ in libraries if _['id'] == library_id), None)
             libraries = [library] if library is not None else []
@@ -229,18 +327,19 @@ class LibraryClient(Client):
         :param library_id: filter for library by library id
 
         :type contents: bool
-        :param contents: True if want to get contents of the library (rather than just the library details).
+        :param contents: whether to get contents of the library (rather
+          than just the library details)
 
         :rtype: dict
         :return: details of the given library
         """
-        return Client._get(self, id=library_id, contents=contents)
+        return self._get(id=library_id, contents=contents)
 
     def _do_upload(self, library_id, **keywords):
         """
         Set up the POST request and do the actual data upload to a data library.
-        This method should not be called directly but instead refer to the methods
-        specific for the desired type of data upload.
+        This method should not be called directly but instead refer to the
+        methods specific for the desired type of data upload.
         """
         folder_id = keywords.get('folder_id', None)
         if folder_id is None:
@@ -256,6 +355,10 @@ class LibraryClient(Client):
             payload["roles"] = keywords["roles"]
         if keywords.get("link_data_only", None) and keywords['link_data_only'] != 'copy_files':
             payload["link_data_only"] = 'link_to_files'
+        payload['tag_using_filenames'] = keywords.get('tag_using_filenames', False)
+        if keywords.get('tags'):
+            payload['tags'] = keywords['tags']
+        payload['preserve_dirs'] = keywords.get('preserve_dirs', False)
         # upload options
         if keywords.get('file_url', None) is not None:
             payload['upload_option'] = 'upload_file'
@@ -275,13 +378,15 @@ class LibraryClient(Client):
             payload["filesystem_paths"] = keywords["filesystem_paths"]
 
         try:
-            return Client._post(self, payload, id=library_id, contents=True,
-                                files_attached=files_attached)
+            return self._post(payload, id=library_id, contents=True,
+                              files_attached=files_attached)
         finally:
             if payload.get('files_0|file_data', None) is not None:
                 payload['files_0|file_data'].close()
 
-    def upload_file_from_url(self, library_id, file_url, folder_id=None, file_type='auto', dbkey='?'):
+    def upload_file_from_url(self, library_id, file_url, folder_id=None,
+                             file_type='auto', dbkey='?',
+                             tags=None):
         """
         Upload a file to a library from a URL.
 
@@ -300,12 +405,21 @@ class LibraryClient(Client):
 
         :type dbkey: str
         :param dbkey: Dbkey
+
+        :type tags: list
+        :param tags: A list of tags to add to the datasets
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the LDDA
         """
         return self._do_upload(library_id, file_url=file_url,
                                folder_id=folder_id, file_type=file_type,
-                               dbkey=dbkey)
+                               dbkey=dbkey,
+                               tags=tags)
 
-    def upload_file_contents(self, library_id, pasted_content, folder_id=None, file_type='auto', dbkey='?'):
+    def upload_file_contents(self, library_id, pasted_content,
+                             folder_id=None, file_type='auto', dbkey='?',
+                             tags=None):
         """
         Upload pasted_content to a data library as a new file.
 
@@ -324,13 +438,21 @@ class LibraryClient(Client):
 
         :type dbkey: str
         :param dbkey: Dbkey
+
+        :type tags: list
+        :param tags: A list of tags to add to the datasets
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the LDDA
         """
         return self._do_upload(library_id, pasted_content=pasted_content,
                                folder_id=folder_id, file_type=file_type,
-                               dbkey=dbkey)
+                               dbkey=dbkey,
+                               tags=tags)
 
     def upload_file_from_local_path(self, library_id, file_local_path,
-                                    folder_id=None, file_type='auto', dbkey='?'):
+                                    folder_id=None, file_type='auto', dbkey='?',
+                                    tags=None):
         """
         Read local file contents from file_local_path and upload data to a
         library.
@@ -350,21 +472,29 @@ class LibraryClient(Client):
 
         :type dbkey: str
         :param dbkey: Dbkey
+
+        :type tags: list
+        :param tags: A list of tags to add to the datasets
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the LDDA
         """
         return self._do_upload(library_id, file_local_path=file_local_path,
                                folder_id=folder_id, file_type=file_type,
-                               dbkey=dbkey)
+                               dbkey=dbkey,
+                               tags=tags)
 
     def upload_file_from_server(self, library_id, server_dir, folder_id=None,
                                 file_type='auto', dbkey='?', link_data_only=None,
-                                roles=""):
+                                roles="", preserve_dirs=False, tag_using_filenames=False,
+                                tags=None):
         """
         Upload all files in the specified subdirectory of the Galaxy library
         import directory to a library.
 
         .. note::
           For this method to work, the Galaxy instance must have the
-          ``library_import_dir`` option configured in the ``config/galaxy.ini``
+          ``library_import_dir`` option configured in the ``config/galaxy.yml``
           configuration file.
 
         :type library_id: str
@@ -374,7 +504,7 @@ class LibraryClient(Client):
         :param server_dir: relative path of the subdirectory of
           ``library_import_dir`` to upload. All and only the files (i.e. no
           subdirectories) contained in the specified directory will be
-          uploaded.
+          uploaded
 
         :type folder_id: str
         :param folder_id: id of the folder where to place the uploaded files.
@@ -393,23 +523,42 @@ class LibraryClient(Client):
 
         :type roles: str
         :param roles: ???
+
+        :type preserve_dirs: bool
+        :param preserve_dirs: Indicate whether to preserve the directory structure when importing dir
+
+        :type tag_using_filenames: bool
+        :param tag_using_filenames: Indicate whether to generate dataset tags
+          from filenames.
+
+          .. versionchanged:: 0.14.0
+            Changed the default from ``True`` to ``False``.
+
+        :type tags: list
+        :param tags: A list of tags to add to the datasets
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the LDDA
         """
         return self._do_upload(library_id, server_dir=server_dir,
                                folder_id=folder_id, file_type=file_type,
                                dbkey=dbkey, link_data_only=link_data_only,
-                               roles=roles)
+                               roles=roles, preserve_dirs=preserve_dirs,
+                               tag_using_filenames=tag_using_filenames,
+                               tags=tags)
 
     def upload_from_galaxy_filesystem(self, library_id, filesystem_paths, folder_id=None,
                                       file_type="auto", dbkey="?", link_data_only=None,
-                                      roles=""):
+                                      roles="", preserve_dirs=False, tag_using_filenames=False,
+                                      tags=None):
         """
         Upload a set of files already present on the filesystem of the Galaxy
         server to a library.
 
         .. note::
           For this method to work, the Galaxy instance must have the
-          ``allow_library_path_paste`` option set to ``True`` in the
-          ``config/galaxy.ini`` configuration file.
+          ``allow_path_paste`` option set to ``true`` in the
+          ``config/galaxy.yml`` configuration file.
 
         :type library_id: str
         :param library_id: id of the library where to place the uploaded file
@@ -435,11 +584,29 @@ class LibraryClient(Client):
 
         :type roles: str
         :param roles: ???
+
+        :type preserve_dirs: bool
+        :param preserve_dirs: Indicate whether to preserve the directory structure when importing dir
+
+        :type tag_using_filenames: bool
+        :param tag_using_filenames: Indicate whether to generate dataset tags
+          from filenames.
+
+          .. versionchanged:: 0.14.0
+            Changed the default from ``True`` to ``False``.
+
+        :type tags: list
+        :param tags: A list of tags to add to the datasets
+
+        :rtype: list
+        :return: List with a single dictionary containing information about the LDDA
         """
         return self._do_upload(library_id, filesystem_paths=filesystem_paths,
                                folder_id=folder_id, file_type=file_type,
                                dbkey=dbkey, link_data_only=link_data_only,
-                               roles=roles)
+                               roles=roles, preserve_dirs=preserve_dirs,
+                               tag_using_filenames=tag_using_filenames,
+                               tags=tags)
 
     def copy_from_dataset(self, library_id, dataset_id, folder_id=None, message=''):
         """
@@ -457,6 +624,9 @@ class LibraryClient(Client):
 
         :type message: str
         :param message: message for copying action
+
+        :rtype: dict
+        :return: LDDA information
         """
         if folder_id is None:
             folder_id = self._get_root_folder_id(library_id)
@@ -465,11 +635,11 @@ class LibraryClient(Client):
         payload['create_type'] = 'file'
         payload['from_hda_id'] = dataset_id
         payload['ldda_message'] = message
-        return Client._post(self, payload, id=library_id, contents=True)
+        return self._post(payload, id=library_id, contents=True)
 
     def get_library_permissions(self, library_id):
         """
-        Get the permessions for a library.
+        Get the permissions for a library.
 
         :type library_id: str
         :param library_id: id of the library
@@ -477,13 +647,26 @@ class LibraryClient(Client):
         :rtype: dict
         :return: dictionary with all applicable permissions' values
         """
-        url = '/'.join([self.gi._make_url(self, library_id), 'permissions'])
-        return Client._get(self, url=url)
+        url = self._make_url(library_id) + '/permissions'
+        return self._get(url=url)
+
+    def get_dataset_permissions(self, dataset_id):
+        """
+        Get the permissions for a dataset.
+
+        :type dataset_id: str
+        :param dataset_id: id of the dataset
+
+        :rtype: dict
+        :return: dictionary with all applicable permissions' values
+        """
+        url = '/'.join((self._make_url(), 'datasets', dataset_id, 'permissions'))
+        return self._get(url=url)
 
     def set_library_permissions(self, library_id, access_in=None,
                                 modify_in=None, add_in=None, manage_in=None):
         """
-        Set the permissions for a library.  Note: it will override all security
+        Set the permissions for a library. Note: it will override all security
         for this library even if you leave out a permission type.
 
         :type library_id: str
@@ -500,8 +683,10 @@ class LibraryClient(Client):
 
         :type manage_in: list
         :param manage_in: list of role ids
-        """
 
+        :rtype: dict
+        :return: General information about the library
+        """
         payload = {}
         if access_in:
             payload['LIBRARY_ACCESS_in'] = access_in
@@ -511,5 +696,38 @@ class LibraryClient(Client):
             payload['LIBRARY_ADD_in'] = add_in
         if manage_in:
             payload['LIBRARY_MANAGE_in'] = manage_in
-        url = '/'.join([self.gi._make_url(self, library_id), 'permissions'])
-        return Client._post(self, payload, url=url)
+        url = self._make_url(library_id) + '/permissions'
+        return self._post(payload, url=url)
+
+    def set_dataset_permissions(self, dataset_id, access_in=None,
+                                modify_in=None, manage_in=None):
+        """
+        Set the permissions for a dataset. Note: it will override all security
+        for this dataset even if you leave out a permission type.
+
+        :type dataset_id: str
+        :param dataset_id: id of the dataset
+
+        :type access_in: list
+        :param access_in: list of role ids
+
+        :type modify_in: list
+        :param modify_in: list of role ids
+
+        :type manage_in: list
+        :param manage_in: list of role ids
+
+        :rtype: dict
+        :return: dictionary with all applicable permissions' values
+        """
+        payload = {}
+        if access_in:
+            payload['access_ids[]'] = access_in
+        if modify_in:
+            payload['modify_ids[]'] = modify_in
+        if manage_in:
+            payload['manage_ids[]'] = manage_in
+        # we need here to define an action
+        payload['action'] = 'set_permissions'
+        url = '/'.join((self._make_url(), 'datasets', dataset_id, 'permissions'))
+        return self._post(payload, url=url)
