@@ -1,8 +1,11 @@
 import os
-from collections import namedtuple
+from typing import (
+    IO,
+    NamedTuple
+)
 
 
-class Bunch(object):
+class Bunch:
     """
     A convenience class to allow dict keys to be represented as object fields.
 
@@ -19,16 +22,12 @@ class Bunch(object):
         return str(self.__dict__)
 
 
-def _file_stream_close(self):
-    """
-    Close the open file descriptor associated with the FileStream
-    object.
-    """
-    self[1].close()
+class FileStream(NamedTuple):
+    name: str
+    fd: IO
 
-
-FileStream = namedtuple("FileStream", ["name", "fd"])
-FileStream.close = _file_stream_close
+    def close(self):
+        self.fd.close()
 
 
 def attach_file(path, name=None):
@@ -50,7 +49,23 @@ def attach_file(path, name=None):
     attachment = FileStream(name, open(path, "rb"))
     return attachment
 
-__all__ = [
+
+def abstractclass(decorated_cls):
+    """
+    Decorator that marks a class as abstract even without any abstract method
+
+    Adapted from https://stackoverflow.com/a/49013561/4503125
+    """
+    def clsnew(cls, *args, **kwargs):
+        if cls is decorated_cls:
+            raise TypeError(f"Can't instantiate abstract class {decorated_cls.__name__}")
+        return super(decorated_cls, cls).__new__(cls)
+
+    decorated_cls.__new__ = clsnew
+    return decorated_cls
+
+
+__all__ = (
     'Bunch',
     'attach_file',
-]
+)
