@@ -3,7 +3,7 @@ This example retrieves details of all the Histories in our Galaxy account and li
 
 Usage: python list_histories.py <galaxy-url> <galaxy-API-key>
 """
-from __future__ import print_function
+
 import sys
 
 from bioblend.galaxy import GalaxyInstance
@@ -29,4 +29,4 @@ else:
     for hist_dict in histories:
         # As an example, we retrieve a piece of metadata (the size) using show_history
         hist_details = gi.histories.show_history(hist_dict['id'])
-        print("{0} ({1}) : {2}".format(hist_dict['name'], hist_details['nice_size'], hist_dict['id']))
+        print(f"{hist_dict['name']} ({hist_details['size']}) : {hist_dict['id']}")
